package clientswing.UI;
import clientswing.model.*;
import clientswing.comunicationsystem.*;
import javax.swing.*;


/**
* Controller  la classe che si occupa di inviare le XRequest al ComunicationSystem e
* prende decisioni circa quale frame visualizzare in base alla XResponse ricevuta.
* Viene utilizzato il pattern SINGLETON attraverso il quale, utilizzando un'instanza prototipa,
* crea nuove instanze tramite la copia di questo prototipo.
* */
public class Controller {

/** istanza di ComunicationSystem*/
  ComunicationSystem cs;
/**prototipo di instanza di Controller*/
  static Controller instance = new Controller();

/**Costruttore della classe Controller*/
  private Controller() {
    try{
      cs = new ComunicationSystem();
    }catch(Exception e){
      throw new RuntimeException(e);
    }
  }
/**ritorna un'istanza di Controller
   * @return Controller instance
   * */
  public static Controller getInstance(){
    return instance;
  }
/**visualizza la form di login*/
  public void showLogin(){
    LoginForm loginForm = new LoginForm();
    loginForm.show();
  }
/**visualizza la form di query*/
  public void showQueryForm(){
    QueryForm queryForm  = new QueryForm ();
    queryForm.show();
  }
/**visualizza il messaggio di risposta di ok o di errore*/
  public void showMessageForm(XResponseMessage xresponseMessage ){
   if (xresponseMessage.getMessage().equals("error")==true)

     JOptionPane.showMessageDialog(null, xresponseMessage.getTypeMessage(), xresponseMessage.getMessage(), JOptionPane.WARNING_MESSAGE);
     else
       JOptionPane.showMessageDialog(null, xresponseMessage.getTypeMessage(), xresponseMessage.getMessage(), JOptionPane.INFORMATION_MESSAGE);

  }
/**visualizza il risultato di una query*/
  public void showResponseForm(XResponseRowSet rowset ){
    ResponseForm responseForm = new ResponseForm(rowset);
    responseForm.show();

  }
 /**Esegue la Login
  * @param XRequestLogin nickName
  * */
 public void executeLogin( XRequestLogin nickName){

  try {
    XResponseMessage resp = (XResponseMessage) cs.execute(nickName);
    // se la risposta del WebServer non  di errore dopo la login
    //allora visualizza la form dei servizi (QueryForm)
    if (!resp.isErrorMessage()) showQueryForm();

  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
 }
/**esegue il logout
  * @return il messaggio del webServer
  * */
 public String executeLogout() throws Exception{

   String logout= cs.execute();
   return logout;
}
/**Esegue la query e se il risultato  un messaggio allora lo visualizza, altrimenti
  * visualizza il risultato.
  * */
 public void executeQuery(XRequestQuery xrequestQuery ){
  try {
    XResponse resp = cs.execute(xrequestQuery);
    //visualizza il risultato della query
    if (resp instanceof XResponseRowSet) showResponseForm( (XResponseRowSet) resp );
    //visualizza il messaggio
    if (resp instanceof XResponseMessage) showMessageForm( (XResponseMessage) resp );


  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
  }


}
