package clientswing.comunicationsystem;



import java.io.*;

import clientswing.model.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.w3c.dom.Element;
import org.xml.sax.*;


/**classe che a partire dalla risposta del server ne fa il parser.
* Viene utilizzata dal Controller che visualizzer la risposta o come messaggio
* oppure come risultato della query*/
public class XMLParser {

  XResponseRowSet xresponse;
  private Document doc;
  private DocumentBuilderFactory factory;
  private DocumentBuilder builder;
  private InputSource source;


  public XMLParser() {
  }

  private void init() throws ParserConfigurationException {
    factory = DocumentBuilderFactory.newInstance();
    factory.setNamespaceAware(true);
    factory.setValidating(false);
    builder = factory.newDocumentBuilder();


  }

/**metodo che a partire dalla stringa xml che invia il server
   * ne fa il parser e ritorna un oggetto di tipo XResponse.*/
  public XResponse parse(String xml)
     throws ParserConfigurationException, IOException, SAXException {
    xresponse = new XResponseRowSet();
    Document doc;
    System.out.print("XML Ricevuto:" + xml);
    init();
    doc = builder.parse( new InputSource( new StringReader(xml)));

    System.out.print("Parsed.. doc:" + doc);

    if (doc.getDocumentElement().getFirstChild().getNodeName()!="message" ){
      //controller fa visualizzare ResponseForm con il risultato della query
      return buildRowSet(doc);
    }
    else
      //controller fa visualizzare un messaggio
      return buildMessage(doc);


  }
/**metodo che a partire dall'oggetto Document creato immagazzina i valori in
   * XResponseRowSet che a sua volta verr usata da Controller per far visualizzare
   * la ResponseForm.
   * @param Document
   * @return XResponseRowSet
   * */
  private XResponseRowSet buildRowSet(Document doc){
    XResponseRowSet  rowSet = new XResponseRowSet();

    Element root = doc.getDocumentElement();
    int i,j,k;

    System.out.println("Doc:" + doc);
    System.out.println("root: " + root);


    Node rowSetNode = findRowsetNode( root);

    System.out.println("RowSet: " + rowSetNode);

    if (rowSetNode == null) return null;

    NodeList rows = rowSetNode.getChildNodes();

    System.out.print("Number of rows: " + rows.getLength());

    for (i=0;i< rows.getLength();i++){
      if (!rows.item(i).getNodeName().equals("#text"))
        rowSet.addRow(readRow(rows.item(i) ) );
    }

    return rowSet;
  }

/**metodo che permette di trovare un Nodo dell'albero DOM*/
  private Node findRowsetNode(Node n){
    Node child;
    for (int i=0;i<n.getChildNodes().getLength();i++){
      child = n.getChildNodes().item(i);
      if (child.getNodeName().equals("rowset")) return child;
    }

    return null;
  }
/**metodo necessario per non far stampare la stringa #test che il DOM
   * assegna come valore ad ogni nodo che crea, inoltre restituisce come risultato
   * un oggetto Row che conterr i valori delle righe.*/
  private Row readRow(Node row){
    Row r = new Row();
    int i;

    System.out.println("Reading row:" + row);
    NodeList columns = row.getChildNodes();

    for (i = 0; i < columns.getLength(); i++) {
      if (!columns.item(i).getNodeName().equals("#text"))
        r.addColumn(readColumn(row.getChildNodes().item(i)));
    }

    return r;
  }
/**metodo che verifica se una colonna contiene un valore
   * @param Node column
      * @return Column column, che conterr i valori delle colonne*/
  private Column readColumn(Node column){
    Column c = new Column("noName");
    int i;

    System.out.println("Reading column" + column);

    for (i = 0; i < column.getChildNodes().getLength(); i++) {
      Node curr = column.getChildNodes().item(i);
      System.out.println("Found: " + curr.getNodeName() + " Value: " + curr.getNodeValue());
      if (curr.getNodeName().equalsIgnoreCase("name")) {
        c.setName(curr.getFirstChild().getNodeValue());
        System.out.println("Setting name to " + curr.getFirstChild().getNodeValue());
      }
      else {
        if (curr.getNodeName().equalsIgnoreCase("value")) {
          c.setValue(curr.getFirstChild().getNodeValue());
          System.out.println("Setting value to " + curr.getFirstChild().getNodeValue());
        }

      }
    }

    return c;
  }


/**metodo che costruisce il messaggio di risposta a partire dal Document creato. */
  private XResponseMessage buildMessage(Document doc){

    Element root = doc.getDocumentElement();
    System.out.println("Doc:" + doc);
    System.out.println("root: " + root);
    String typeMessage = root.getFirstChild().getAttributes().getNamedItem("result").getNodeValue();
    String message = root.getFirstChild().getFirstChild().getNodeValue();
    System.out.println("Message: " + message);
    System.out.println("TypeMessage: " + typeMessage);

    XResponseMessage mesText = new XResponseMessage(typeMessage,message);

    return mesText;

  }

}
